;******************************************************************************
;
;    This program is free software; you can redistribute it and/or modify
;    it under the terms of the GNU General Public License as published by
;    the Free Software Foundation; either version 2 of the License, or
;    (at your option) any later version.
;
;    This program is distributed in the hope that it will be useful,
;    but WITHOUT ANY WARRANTY; without even the implied warranty of
;    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;    GNU General Public License for more details.
;
;    You should have received a copy of the GNU General Public License
;    along with this program; if not, write to the Free Software
;    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
;
;    I (Simon Wood) can be contacted via email - unilink@mungewell.org
;    or via post - 14 Kirklands Close, Baildon, West Yorkshire, England.
;
;******************************************************************************
; Various responses to the commands
; These are clocked out by the master when it is ready

Respond_Anyone:
	movf	MyId, 1				; Test if we've got an ID

	btfsc	STATUS, Z
	goto	Respond_Anyone_main		;; if MyId=0, then get ID.

	btfss	MyIdCont_Flag, 0		;; even if you've already had MyId,
						;; it's out of date..
	goto	ReceiveBreak			;; 

	clrf	MyId				;; case out of date, get ID again.
	clrf	Appoint_Now			;; set Not my turn.

	IFDEF	Second_Anyone
;;---
	movlw	0x01			;; 
	movwf	FirstAnyone		;; set FirstAnyone =1
;;---
	ENDIF	;<Second_Anyone>


Respond_Anyone_main:
	btfss	OUT_PORT, BusOnBit		; Test for BusOn being high
	goto	ReceiveBreak			;; (BusOn=OFF), Abort.

	IFDEF	Second_Anyone
;;---
	decfsz	FirstAnyone,1		;; FirstAnyone -1 -> FirstAnyone

	goto	Respond_Anyone_main2	;; (case Not 0), 

	goto	ReceiveBreak		;; (case 0), 
					;; skip responding to the first anyone
;;---
	ENDIF	;<Second_Anyone>

Respond_Anyone_main2:
	clrf	MyIdCont_Flag

	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'I'm a .....'
	
	call	SendByte

	movf	MyGroup, 0
	call	SendByte

Respond_Anyone2:
	call	Send_Anyone
	
	bsf	Appoint_Now, 0			;; set Appoint_Now=1 (It's my turn..)

Respond_End:
	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte

	goto	ReceiveBreak			; Done, wait for next command

;--------

Respond_Appoint:
	btfss	Appoint_Now, 0			;; My turn?
	goto	ReceiveBreak			; (not my turn) goto RB.

	movf	Rad, 0				; Store allocated ID
	movwf	MyId

	movf	Cmd2, 0				; Store bit position 
	movwf	MyBit				; (for SlavePoll response)

Respond_App2:
	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'OK I'm ID=X'
	call	SendByte

	movf	MyId, 0
	call	SendByte

;;	btfss	OUT_PORT, CdNotMdBit		; unconnected = CD
;;--					
	movf	MyGroup, 0
	xorlw	0xd0			
	btfsc	STATUS, Z
;;--
	goto	Respond_App3			; same as 'Anyone' (case MD)

						;; (case CD)
	movlw	Appoint_0			; different response
	call	SendByte

	movlw	Appoint_1
	call	SendByte

	call	Checksum_mid			;; <display.asm>

	movlw	Appoint_2
	call	SendByte

	movlw	Appoint_3
	call	SendByte

	movlw	Appoint_4
	call	SendByte

	movlw	Appoint_5
	call	SendByte

	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte
	
	clrf	MyIdCont_Flag			;;
	clrf	Appoint_Now			;;

	goto	ReceiveBreak			; Done, wait for next command


Respond_App3:
	call	Send_Anyone			;; same as respond anyone.

	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte
	
	clrf	MyIdCont_Flag			;;
	clrf	Appoint_Now			;;

	goto	ReceiveBreak			; Done, wait for next command


Send_Anyone:
	movlw	Anyone_0
	call	SendByte

	movlw	Anyone_1
	call	SendByte

	call	Checksum_mid			;; <display.asm>

	movlw	Anyone_2
	call	SendByte

	movlw	Anyone_3
	call	SendByte

	movlw	Anyone_4
	call	SendByte

	movlw	Anyone_5
	call	SendByte

	return

Respond_Hello:
	incf	CnfChk_Count, 1		;; CnfChk_Count+1 -> CnfChk_Count
	movlw	0x03			;; threshold(0x03) -> W
	subwf	CnfChk_Count, 0		;; CnfChk_Count - 0x03 -> W
	btfsc	STATUS, C		;;
	goto	Respond_Hello2		;;(1:positive or zero) You've missed Me!!

	movf	MyId, 1				; Test for ID
	btfss	STATUS, Z
	goto	ReceiveBreak			; keep quiet if we have...

Respond_Hello2:
	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'You've missed me!!'
	call	SendByte

	movlw	0x18
	call	SendByte

	movlw	0x04
	call	SendByte

	movlw	0x00
	call	SendByte

	goto	Respond_End

;--------

Respond_Ping:
	movf	MyId, 0				; Check it's the right ID
	subwf	Rad, 0
	btfss	STATUS, Z
	goto	ReceiveBreak			; Done, wait for next command

	clrwdt					; Reset the watchdog

	bsf	MyIdCont_Flag, 0		;; set MyIdCont_Flag

	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'I'm playing'
	call	SendByte

	movf	MyId, 0
	call	SendByte

	movlw	0x00
	call	SendByte

	movlw	0x80				; 'Idle'
	
	movf	Display, 1			; test if we're active
	btfss	STATUS, Z
	movlw	0x00				; 'Playing'

	call	SendByte

	movf	Checksum, 0			; Send Checksum
	call	SendByte
	
	movlw	0x00
	call	SendByte

	clrf	CnfChk_Count			;; (T)

;;	movf	Display, 0
;;	andlw	0x7f				; strip off high nyble
;;	xorlw	0x00				; Test to see if we've got 
;;						; something to display
;;	btfss	STATUS, Z
;;	goto	IssueSlaveBreak
	goto	ReceiveBreak			; wait for next command


;******************************************************************************
; Response to Master Poll Command

Respond_MasterPoll:
	btfss	NeedToTalk, 0		;;Test wheather I need to talk

	goto	ReceiveBreak		;; case "OFF", abort	

	clrf	NeedToTalk
	bcf	ISB_flag, Res_ISB	;; set Reserve_ISB bit = 0

	movf	Display, 0
	xorlw	0x00				; Test if we're active
	btfsc	STATUS, Z
	goto	ReceiveBreak			; wait for next command
	
	clrf	Checksum 			; Clear checksum

	movlw	0x10				; 'Something has changed!'
	call	SendByte

	movlw	0x18				; is to a broadcast address
	call	SendByte

	movlw	0x82
	call	SendByte

	movlw	0x00				; MyBit is 0x0? or 0x1?
	call	Bit_Frig
	call	SendByte

	call	Checksum_mid

	movlw	0x20				; MyBit is 0x2? or 0x3?
	call	Bit_Frig
	call	SendByte

	movlw	0x40				; MyBit is 0x4? or 0x5?
	call	Bit_Frig
	call	SendByte

	movlw	0x60				; MyBit is 0x6? or 0x7?
	call	Bit_Frig
	call	SendByte

	movlw	0x80				; MyBit is 0x8? or 0x9?
	call	Bit_Frig
	call	SendByte
	
	goto	Respond_End			; to do checksum and error byte

;******************************************************************************








